#!/bin/sh

####################################
#
# Centrify DirectAudit 
# dashellfix script
# Copyright (C) 2006-2020 Centrify Corp
#
####################################
# Usage:
#    
#     usage: dashellfix [-c] [-d]
#     options:
#       --create, -e     Creates DirectAudit substitute shell links.
#       --destroy, -d    Destroys DirectAudit substitute shell links.
#       --verbose, -v    Verbose output (off by default)
#       --help, -h       Output this usage statement.
#
# Requires that:
#   - cat, rm, ln and sed be in callers path
#   - caller is root
####################################

# Global vars
NAME=dashellfix
USAGE="$0 { [--create | -c] | [--destroy | -d] } [--verbose | -v]"

# Add custom or other shells here, dups are okay (ignored)
DEFAULT_SHELL_LIST="/bin/bash /bin/jsh /bin/pfcsh /bin/pfsh /bin/tcsh /sbin/jsh
                    /usr/bin/bash /usr/bin/jsh /usr/bin/pfcsh /usr/bin/pfsh /usr/bin/tcsh
                    /bin/csh /bin/ksh /bin/pfksh /bin/sh /bin/zsh /sbin/sh /usr/bin/csh
                    /usr/bin/ksh /usr/bin/pfksh /usr/bin/sh /usr/bin/zsh /sbin/sh
                    /bin/ash /bin/bsh /bin/zsh /bin/tcsh"

# This will be filled in later by GetEtcShells
ETC_SHELL_LIST=""

# Default action is to output usage
ACTION=help
# Default verbosity is off
VERBOSE=0
# Base directory/name for links
BASELINK="/usr/bin/dash"


#### Helper functions

# Get any/all shells from /etc/shells if it exists
GetEtcShells()
{
    if [ -f "/etc/shells" ]; then
        ETC_SHELL_LIST=`/bin/cat /etc/shells`
    fi
}

# If verbosity is on, output the passed arguments
Verbose()
{
    if [ "$VERBOSE" = "1" ]; then
        echo "$@"
    fi
}

# Use our own version of 'which' to reduce dependencies
# on external shells.
Which()
{
    IFS=":"
    for path in $PATH; do
        if [ -x $path/$1 ]; then
            echo "$path/$1"
            return
        fi
    done
    echo ""
}

# If $1 is not == $2 then output $3 and exit with
#   status 1.
Error()
{
    echo "$@"
    Verbose "Exiting on failure"
    exit 1
}

# Make sure we're root if we can
CheckRoot()
{
    if [ -z "$UID" ]; then
        if [ -x /usr/xpg4/bin/id ]; then
            UID=`/usr/xpg4/bin/id -u`
        else
            UID=`id -u`
        fi
    fi
    if [ -n $UID -a "$UID" -ne "0" ]; then
        Error "This script must be executed by root."
    else
        Verbose "Unable to determine current UID, this script must be executed as root, trying to continue operation"
    fi
}

#### Define functions for the different modes of operation

# We need to create links for all possible
#   shells such that:
#
# Shell: /path/to/shell
# /usr/bin/dash_path_to_shell -> /path/to/shell
#
# We get possible shells from the combination
#   of a default list, and anything in
#   /etc/shells (if available)
#
CreateShellLinks() 
{
    GetEtcShells
    SHELL_LIST="$DEFAULT_SHELL_LIST $ETC_SHELL_LIST"

    # For each shell
    for shell in $SHELL_LIST
    do
        # Make sure the shell exists
        if [ -f "$shell" ]; then
            # Build the name of the link
            LINK_NAME="${BASELINK}`echo $shell | sed -e 's/\//_/g'`"
            # Make sure a link by the same name isn't already there
            if [ ! -h $LINK_NAME ]; then
                Verbose "Creating link $LINK_NAME to $shell"
                ln -s $shell $LINK_NAME > /dev/null 2>&1
                if [ ! "$?" = "0" ]; then
                    Error "Unable to create link $LINK_NAME to $shell, do you have permission to do so?"
                else
                    # We created a link, add the original and the link to /etc/shells
                    echo $shell >> /etc/shells
                    echo $LINK_NAME >> /etc/shells
                fi
            else
                Verbose "$LINK_NAME already exists, skipping."
            fi
        fi
    done

    # We may have added things to /etc/shells, re-create it with duplicates removed
    SORT=`which sort`
    if [ ! "$SORT" = "" ]; then
        cat /etc/shells | $SORT -u > /etc/shells.dashellfix 2>/dev/null
        if [ $? = 0 ]; then
            mv -f /etc/shells.dashellfix /etc/shells
        fi
    fi
}

# We need to destroy all /usr/bin/dash_* links
DestroyShellLinks()
{
    LINK_LIST=`ls /usr/bin/dash_* 2>/dev/null`

    if [ "$LINK_LIST" = "" ]; then
        Verbose "No linked shells found, done."
    else
        # For each shell
        for shell in $LINK_LIST
        do
            # If its a link, break it
            if [ -h $shell ]; then
                Verbose "Breaking link $shell"
                rm -f $shell > /dev/null 2>&1
                if [ ! "$?" = "0" ]; then
                    Error "Unable to break link $shell, do you have permission to do so?" 
                fi
            else
                # otherwise just output if verbose is on
                Verbose "$shell is not a link, skipping"
            fi
        done
        # We've removed all links, now remove them from /etc/shells
        cat /etc/shells | grep -v "/usr/bin/dash_" > /etc/shells.dashellfix 2>/dev/null
        if [ $? = 0 ]; then
            mv -f /etc/shells.dashellfix /etc/shells
        fi
    fi
}

### Main processing starts here

# Make sure we are root
CheckRoot

# Make sure we are not running on a system with cdash
if [ -x /usr/share/centrifydc/bin/cdash ]; then # This must be cdash. The prefix makes no effect.
    Error "Error: DirectAudit shell detected, this script is intended for use on systems without DirectAudit installed."
fi

# Make sure we have an argument
if [ ! $# -gt 0 ]; then
    echo $USAGE
    exit
fi

# Process arguments
while [ -n "$1" ]
do
    case $1 in
        -c) ACTION=create ;;
        --create) ACTION=create ;;
        -d) ACTION=destroy ;;
        --destroy) ACTION=destroy ;;
        -h) ACTION=help ;;
        --help) ACTION=help ;;
        -v) VERBOSE=1 ;;
        --verbose) VERBOSE=1 ;;
        *)  echo $USAGE
            exit ;;
    esac      
    shift
done

echo "Action: $ACTION"

# Proceed according to ACTION
case $ACTION in
    create)
            CreateShellLinks
            ;;
    destroy)
            DestroyShellLinks
            ;;
    help)
            echo $USAGE
            exit
            ;;
esac

