#!/bin/sh /usr/share/centrifydc/perl/run
#
##############################################################################
#
# Copyright (C) 2008-2020 Centrify Corporation. All rights reserved.
#
# machine/user mapper script to do adreload, adflush, restart adclient, etc.
#
# Several mapper scripts need to do adreload/adflush if configuration changed.
# To prevent multiple adreload/adflush, these mapper scripts will not run
# adreload/adflush directly. Instead, they'll create:
#       /var/centrifydc/reg/do_adreload
#       /var/centrifydc/reg/do_adflush
#       /var/centrifydc/reg/do_restart_adclient
# and this mapper script will do the actual reload/flush if these files exist.
#
#
# Parameters: <map|unmap> [username] mode
#   map|unmap   action to take
#   username    username (not used)
#   mode        mode (not used)
#
# Exit value:
#   0   Normal
#   1   Error
#   2   usage
#
##############################################################################

use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;
use CentrifyDC::GP::Args;
use CentrifyDC::GP::General qw(:debug RunCmd CreateFile);
use CentrifyDC::GP::CimsPath qw($CIMS_USR $CIMS_INSTALL);

my $DO_RESTART_ADCLIENT = '/var/centrifydc/reg/do_restart_adclient';
my $DO_RESTART_ADCLIENT_FOR_WATCHDOG = '/var/centrifydc/do_restart_adclient';
my $DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE = '/var/centrifydc/reg/do_restart_expire';
my $DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE_FOR_WATCHDOG = '/var/centrifydc/do_restart_expire';
my $DO_STOP_ADCLIENT = '/var/centrifydc/reg/do_stop_adclient';
my $DO_STOP_ADCLIENT_FOR_WATCHDOG = '/var/centrifydc/do_stop_adclient';
my $DO_ADRELOAD         = '/var/centrifydc/reg/do_adreload';
my $DO_ADFLUSH          = '/var/centrifydc/reg/do_adflush';
my $DO_ADFLUSH_FORCE    = '/var/centrifydc/reg/do_adflush_force';
my $DO_ADFLUSH_CLOUD    = '/var/centrifydc/reg/do_adflush_cloud';
my $DO_ADFLUSH_OBJECTS  = '/var/centrifydc/reg/do_adflush_objects';
my $DO_ADFLUSH_TRUSTS   = '/var/centrifydc/reg/do_adflush_trusts';
my $DO_DSRELOAD         = '/var/centrifydc/reg/do_dsreload';
my $DO_DSFLUSH          = '/var/centrifydc/reg/do_dsflush';
my $DO_SARESTART        = '/var/centrifydc/reg/do_sarestart';
my $DO_RESTART_LOGINWINDOW = '/var/centrifydc/reg/do_restart_loginwindow';
my $DO_DARELOAD         = '/var/centrifydc/reg/do_dareload';

my $CMD_ADRELOAD         = "$CIMS_USR/sbin/adreload";
# The flush just expires the entire cache, this way we will not lose the users cached creds.
# Losing the users creds is BAD in a laptop system since they are in disconnected mode often.
my $CMD_ADFLUSH          = ["$CIMS_USR/sbin/adflush", '-e'];
my $CMD_ADFLUSH_FORCE    = ["$CIMS_USR/sbin/adflush", '-f'];

my $CMD_ADFLUSH_CLOUD    = ["$CIMS_USR/sbin/adflush", '-c'];
my $CMD_ADFLUSH_OBJECTS  = ["$CIMS_USR/sbin/adflush", '-o'];
my $CMD_ADFLUSH_TRUSTS  = ["$CIMS_USR/sbin/adflush", '-t'];

# reload should be sufficient, but is not (yet)
my $CMD_DSRELOAD         = ["$CIMS_INSTALL/bin/dsconfig", 'reload'];
my $CMD_DSFLUSH          = ["$CIMS_INSTALL/bin/dsconfig", 'flush'];
my $CMD_RESTART_LOGINWINDOW = ['/usr/bin/killall', '-HUP', 'loginwindow'];
my $CMD_RESTART_SECURITYAGENT = ['/usr/bin/killall', '-HUP', 'SecurityAgent'];
my $CMD_RESTART_AUTHORIZATIONHOST = ['/usr/bin/killall', '-HUP', 'authorizationhost'];
# It is sufficient to reload DirectAudit settings, when the centrifyda.conf has been changed.
my $CMD_DARELOAD         = '/usr/sbin/dareload';

sub RestartLoginWindow();


# >>> SUB >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

#
# restart loginwindow
#
# Mac 10.6 or later: kill SecurityAgent
#
#   return: 1       - successful
#           undef   - failed
#
sub RestartLoginWindow()
{
    DEBUG_OUT("Restart SecurityAgent");
    my $rc = RunCmd($CMD_RESTART_SECURITYAGENT);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot restart SecurityAgent");
        return undef;
    }

    return 1;
}

# >>> MAIN >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

my $args = CentrifyDC::GP::Args->new();

$args->isMap() or exit(0);

if (-e $DO_ADRELOAD && ! -e $DO_RESTART_ADCLIENT && ! -e $DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE)
{
    DEBUG_OUT("Do adreload");
    my $rc = RunCmd($CMD_ADRELOAD);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adreload");
    }
    unlink($DO_ADRELOAD);
}

if (-e $DO_ADFLUSH)
{
    DEBUG_OUT("Do adflush");
    my $rc = RunCmd($CMD_ADFLUSH);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adflush");
    }
    unlink($DO_ADFLUSH);
}

if (-e $DO_ADFLUSH_FORCE)
{
    DEBUG_OUT("Do adflush -f");
    my $rc = RunCmd($CMD_ADFLUSH_FORCE);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adflush -f");
    }
    unlink($DO_ADFLUSH_FORCE);

    # -f covers -o
    unlink($DO_ADFLUSH_OBJECTS);
}

if (-e $DO_ADFLUSH_CLOUD)
{
    DEBUG_OUT("Do adflush -c");
    my $rc = RunCmd($CMD_ADFLUSH_CLOUD);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adflush -c");
    }
    unlink($DO_ADFLUSH_CLOUD);
}

if (-e $DO_ADFLUSH_OBJECTS)
{
    DEBUG_OUT("Do adflush -o");
    my $rc = RunCmd($CMD_ADFLUSH_OBJECTS);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adflush -o");
    }
    unlink($DO_ADFLUSH_OBJECTS);
}

if (-e $DO_ADFLUSH_TRUSTS)
{
    DEBUG_OUT("Do adflush -t");
    my $rc = RunCmd($CMD_ADFLUSH_TRUSTS);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adflush -t");
    }
    unlink($DO_ADFLUSH_TRUSTS);
}

if (-e $DO_DSRELOAD)
{
    DEBUG_OUT("Do dsreload");
    my $rc = RunCmd($CMD_DSRELOAD);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do dsreload");
    }
    unlink($DO_DSRELOAD);
}

if (-e $DO_DSFLUSH)
{
    DEBUG_OUT("Do dsflush");
    my $rc = RunCmd($CMD_DSFLUSH);
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do dsflush");
    }
    unlink($DO_DSFLUSH);
}

if (-e $DO_SARESTART)
{
    DEBUG_OUT("Do sarestart (not implemented)");

    unlink($DO_SARESTART);
}

if ($args->mode() eq 'login')
{
    if (-e $DO_RESTART_LOGINWINDOW)
    {
        my $system = `uname -s`;
        if (! defined($system))
        {
            ERROR_OUT("Cannot get system name");
        }
        else
        {
            chomp $system;
            if ($system eq 'Darwin')
            {
                my $rc = RestartLoginWindow();
                if (! $rc)
                {
                    ERROR_OUT("Cannot restart loginwindow");
                }
            }
        }
        unlink($DO_RESTART_LOGINWINDOW);
    }
}

if (-e $DO_STOP_ADCLIENT)
{
    INFO_OUT("Schedule stop of adclient");

    CreateFile("$DO_STOP_ADCLIENT_FOR_WATCHDOG", {mode => 0600});

    unlink($DO_STOP_ADCLIENT);
    unlink($DO_RESTART_ADCLIENT);
    unlink($DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE);
    unlink($DO_ADRELOAD);
}

if (-e $DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE)
{
    INFO_OUT("Schedule restart of adclient and expire of cache");

    CreateFile("$DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE_FOR_WATCHDOG", {mode => 0600});

    unlink($DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE);
    unlink($DO_RESTART_ADCLIENT);
    unlink($DO_ADRELOAD);
}

if (-e $DO_RESTART_ADCLIENT)
{
    INFO_OUT("Schedule restart of adclient");

    CreateFile("$DO_RESTART_ADCLIENT_FOR_WATCHDOG", {mode => 0600});

    unlink($DO_RESTART_ADCLIENT);
    unlink($DO_ADRELOAD);
}

if (-e $DO_DARELOAD)
{
    if (-x $CMD_DARELOAD)
    {
        DEBUG_OUT("Do dareload");
        my $rc = RunCmd($CMD_DARELOAD);
        if (!defined($rc) or $rc ne '0')
        {
            ERROR_OUT("Cannot do dareload");
        }
    }
    else
    {
        ERROR_OUT("$CMD_DARELOAD not exist or not executable");
    }
    unlink($DO_DARELOAD);
}
