#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2015-2020 Centrify Corporation. All rights reserved.
#
# This script is used to modify slapd.conf
#
# arguments:
#       please see sub Usage
# return:
#       0 if succeed
#       1 if meet error
#
# Main logic of script:
# 1. If conf file does not exist but the original conf file ldapproxy.slapd.conf
#       exits. Create conf file /etc/centrify/openldap/slapd.conf from origin.
#    If neither slapd.conf nor ldapproxy.slapd.conf is found, then we consider
#       ldapproxy server package is not installed on this machine. We have
#       nothing to do in this case.
# 2. Back up the original slapd.conf before edit.
# 3. Check and update the slapd.conf appropriately. Script will handle the
#    following issues:
#    a. Support anonymous bind controlled by centrifydc.conf. Default disable.
#    b. Suffix would be fixed to the joined domain name.
#    c. Missing required settings will be wrapped and fixed in proper place.
#    d. For simplicity, duplicated lines for the same required setting will be 
#       removed from slapd.conf.
#       For required settings, we will set them using our desired values. But
#       we will not modify other settings as leave them as is
#       (For settings that appear more than once, the last appearance in the
#       slapd.conf file is used.)
# 4. Restart slapd server if conf file is changed and server is running.
#
# NOTE, slapd.conf consists of three types of configuration information:
# global, backend specific, and database specific. Global information is
# specified first, followed by information associated with a particular backend
# type, which is then followed by information associated with a particular
# database instance. And the sequence of the three components cannot be changed.
# Otherwise slapd server will fail to start.
#
# Actually, If a line begins with whitespace, it is considered a continuation
# of the previous line (even if the previous line is a comment). But luckily,
# our Centrify required settings would not be in this case.
##############################################################################

use warnings;
use strict;
use lib "/usr/share/centrifydc/perl";
use lib "/usr/share/centrifydc/scripts";
use CentrifyDC::Include;
use File::Basename;
use File::stat;
use CentrifyDC::Platform qw(GetPlatform OS ARCH);
use Getopt::Long qw(GetOptions);

use autoedit_common_utils qw(loadSetting getValueList);
use autoedit_file_lock;
use CentrifyDC::GP::General qw(:debug :system);
use CentrifyDC::GP::CimsPath qw($CIMS_USR $CIMS_INSTALL);

# >>> DATA >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

use constant DEFAULT_CFG_FILE => "/etc/centrifydc/openldap/slapd.conf";
use constant LDAPPROXY_CFG_FILE => "/etc/centrifydc/openldap/ldapproxy.slapd.conf";
use constant PROP_LDAPPROXY_REQUIRE_AUTHC => "ldapproxy.require.authc";

my $g_cfg_name = DEFAULT_CFG_FILE;

# Is any setting in slapd.conf has been changed?
my $g_is_changed = 0;
my $g_cdcversion;
my $g_slapdversion;

# To remember the line number of global directives and database specific directives in slapd.conf.
# So we can insert missing required Centrify-ldapproxy settings in right order.
# The backend specific directives will be insert right before database specific. 
my %g_global_index;
my $g_database_index;

# Markers to wrap those settings we add into slapd.conf.
my $SLAPD_CONF_START_PATTERN          = qr/### START CentrifyDC ldapproxy generated slapd.*settings START ###/;
my $SLAPD_CONF_END_PATTERN            = qr/###  END  CentrifyDC ldapproxy generated slapd.*settings  END  ###/;
my $SLAPD_CONF_GLOBAL_START_MARKER    = "### START CentrifyDC ldapproxy generated slapd global settings START ###";
my $SLAPD_CONF_GLOBAL_END_MARKER      = "###  END  CentrifyDC ldapproxy generated slapd global settings  END  ###";
my $SLAPD_CONF_BACKEND_START_MARKER   = "### START CentrifyDC ldapproxy generated slapd backend specific settings START ###";
my $SLAPD_CONF_BACKEND_END_MARKER     = "###  END  CentrifyDC ldapproxy generated slapd backend specific settings  END  ###";
my $SLAPD_CONF_DATABASE_START_MARKER  = "### START CentrifyDC ldapproxy generated slapd database specific settings START ###";
my $SLAPD_CONF_DATABASE_END_MARKER    = "###  END  CentrifyDC ldapproxy generated slapd database specific settings  END  ###";
my $SLAPD_CONF_WARNING_MESSAGE = "#### Auto-generated by CentrifyDC - DO NOT EDIT THIS BLOCK!  ####";

use constant SLAPD_GLOBAL_SCHEMAS    => qw(core cosine ad nis solaris);
# "argsfile sizelimit timelimit" are global directives, but we will override them in backend specific directives.
use constant SLAPD_BACKEND_SPECIFIC  => qw(backend pidfile argsfile modulepath moduleload cdctranslation require sizelimit timelimit);
use constant SLAPD_DATABASE_SPECIFIC => qw(database suffix);
my $g_req_conf = {
    'include' => {
        'core'    => "/etc/centrifydc/openldap/schema/core.schema",
        'cosine'  => "/etc/centrifydc/openldap/schema/cosine.schema",
        'ad'      => "/etc/centrifydc/openldap/schema/ad.schema",
        'nis'     => "/etc/centrifydc/openldap/schema/nis.schema",
        'solaris' => "/etc/centrifydc/openldap/schema/solaris.schema",
    },

    # Server Configuration entries
    'pidfile'   => "/var/run/slapd.pid",
    'argsfile'  => "/var/run/slapd.args",
    'modulepath'=> "$CIMS_INSTALL/libexec/openldap",
    'moduleload'=> "libback_centrifydc.so",
    'backend'   => "centrifydc",

    # Translate RFC2307 requests entries
    'cdctranslation' => "/etc/centrifydc/openldap/rfc2307.map",

    # slapd configuration entries
    # require: Specify a set of conditions to require (default none).
    #          Accept multiple values. e.g. "suffix     authc,LDAPv3,SASL,bind,strong"
    # suffix:  Specify the DN suffix of queries that will be passed to this backend database.
    #          Multiple suffix lines can be given and at least one is required for each database definition.
    'require'   => "authc",            
    'sizelimit' => "unlimited",
    'timelimit' => "unlimited",
    'database'  => "centrifydc",
    'suffix'    => "DC=DOMAIN,DC=COM", 
};

# Define the must-have settings which are not found from slapd.conf
my $g_must_have_entires;
# Define the added settings which would be wrapped in Centrify marker.
my @g_wrapped_entries;

sub Usage;
sub InitMustHaveConfigEntries();
sub GetConfigurationFromLine($);
sub LoadSystemConfiguration();
sub GenerateSettingLine($$);
sub DeleteMemberFromArrayRef($$);
sub DeleteMembersFromArrayRef($$);
sub CheckSlapdConfiguration();
sub IsCDCSolelyUpgraded();
sub RestartSlapdService();

# >>> SUB >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>$

sub Usage
{
    warn << "END";
    This script is used to modify slapd.conf.
    Usage:
    --cfg_file          the full path of slapd.conf
END
    exit 1;
}

#
# Initialize the slapd must-have settings for adclient from hash $g_req_conf.
# Also get the joined domain name.
#
#   return -     0: success
#          - other: error
sub InitMustHaveConfigEntries()
{
    # We only need to initialize the global data once.
    return 0 if (defined($g_must_have_entires));
    $g_must_have_entires = {};

    # Load global directives
    foreach my $key (SLAPD_GLOBAL_SCHEMAS)
    {
        $g_must_have_entires->{$key} = 1;
    }

    # Load backend specific directives
    foreach my $key (SLAPD_BACKEND_SPECIFIC)
    {
        $g_must_have_entires->{$key} = 1;
    }

    # Load database specific directives
    foreach my $key (SLAPD_DATABASE_SPECIFIC)
    {
        $g_must_have_entires->{$key} = 1;
    }

    # Specify if slapd server needs to support anonymous bind.
    # Sub getValueList(propName) treats all property's value(s) as a list and
    # return the value(s) as an array.
    # For single value property, sub stores the value in the first element of array.
    my $require_authc = getValueList(PROP_LDAPPROXY_REQUIRE_AUTHC);
    if ($require_authc && defined($require_authc->[0]))
    {
        DEBUG_OUT("Get property %s from centrifydc.conf: [%s]", PROP_LDAPPROXY_REQUIRE_AUTHC, $require_authc->[0]);
        $g_must_have_entires->{'require'} = 0 if ($require_authc->[0] =~ /\bfalse\b/s);
    }

    # The value of setting "suffix" should be joined domain name.
    # loadSetting() will return undef if machine is not joined to any domain.
    # Since no /var/centrifydc/kset.domain is found.
    my $domain_name = loadSetting('domain');
    if (defined($domain_name))
    {
         # domain name loaded from kset file will be in format: "example.com"
         # We want the: "DC=EXAMPLE,DC=COM"
         my $dn = shift @$domain_name;
         DEBUG_OUT("Machine is joined to domain [$dn]");
         $dn =~ s/\./,DC=/g;
         $g_req_conf->{'suffix'} = "DC=" . "\U$dn";
    }
    else
    {
        DEBUG_OUT("Machine is not joined to any domain.");
    }

    # Setting 'moduleload' would be diff in AIX and HPUX-PA platform.
    my $platform = GetPlatform();
    if ($platform->getOS() eq OS->{AIX})
    {
        $g_req_conf->{'moduleload'} = "$CIMS_INSTALL/libexec/openldap/libback_centrifydc.a(libback_centrifydc.so.0)"
    }
    elsif ($platform->getOS() eq OS->{HPUX} && $platform->getArch() eq ARCH->{PA})
    {
        $g_req_conf->{'moduleload'} = "libback_centrifydc.sl";
    }
    
    return 0;
}

# 
# Parse the setting line and get the key and value.
# return undef for comment and blank lines.
#
#   $_[0] - $line: the setting line.
#
#   return - ($key, $value): the key and value of the slapd setting.
#          - undef         : invalid setting line.
sub GetConfigurationFromLine($)
{
    my ($line) = shift;
    chomp $line;
    # Comment line
    return undef if ($line =~ /^\s*#/);
    # Blank line.
    return undef if ($line =~ /^\s*$/);

    # Remove the comment after valid setting.
    $line =~ s/(.*)#.*/$1/g;

    # The valid setting lines would be:
    # include         /etc/centrifydc/openldap/schema/cosine.schema
    # argsfile        /var/run/slapd.args
    $line =~ /([^\s]+)\s*(.*)/;
    my ($key, $value) = ($1, $2);
    $value =~ s/\s*$//g;

    return ($key, $value);
}

#
# Load and process the contents from slapd.conf.
#
sub LoadSystemConfiguration()
{
    my $cfg_lines = ReadFile("$g_cfg_name");
    if (!defined $cfg_lines)
    {
        # ReadFile already handles the log. No need log now.
        return undef;
    }
    my @cfg_lines = split "\n", $cfg_lines;
    my @new_lines = ();
    my $req_conf = {};
    my $is_marker_line = 0;
    my $omitted_line;
    my $line_num = 0;
    while (defined(my $line = shift @cfg_lines))
    {
        $omitted_line = 0;
        my ($key, $value) = GetConfigurationFromLine($line);
        next if (!defined($key));

        # We store the general required setting as $g_req_conf->{'name'}.
        # In case the setting: 
        # "include   /<full path>/name.schema"
        # "include   /<full path>/name2.schema"
        # Since keyword 'include' could not be a valid key in hash,
        # We store the required schemas settings as $g_req_conf->{'include'}->{<name>};
        my $is_include_schema_setting = 0;
        if ($key eq 'include')
        {
            $key = basename($value, '.schema');
            $req_conf = $g_req_conf->{'include'};
            $is_include_schema_setting = 1;
        }
        else
        {
            $req_conf = $g_req_conf;
        }

        # Just keep the non-required settings added by end users.
        next if (!defined($req_conf->{$key}));

        # Get all the line numbers of required schemas.
        $g_global_index{$key} = $line_num if ($is_include_schema_setting && !$is_marker_line);
        # Get the first line number of databases.
        if ($key eq "database")
        {
            $g_database_index = $line_num if (!defined($g_database_index && !$is_marker_line));
        }

        # - No such $key in $g_must_have_entires if entry has already been checked/updated
        # - else $key has been not checked/updated.
        #   - if setting is disabled explicitly $g_must_have_entires->{$key} = 0
        #   - else $g_must_have_entires->{$key} = 1
        # So if this line including required setting is not in the hash, it's a line including duplicated setting.
        # And we want to remove all duplicated settings in sldapd.conf
        if (!defined($g_must_have_entires->{$key}))
        {
            TRACE_OUT("Remove duplicated setting: [$line]");
            # Delete this line from conf file.
            $omitted_line = 1;

            # Set the flag so we will update the conf file.
            $g_is_changed = 1;
            next;
        }

        # Comment the setting if it's been explicitly disabled by cdc.conf
        # e.g. ldapproxy.require.authc: false
        if (defined($g_must_have_entires->{$key}) && !$g_must_have_entires->{$key})
        {
            # Just comment the setting line so slapd server will use a default setting.
            # e.g. "# require authc"
            # default is "require none"
            $line = "# $line";
            # Remove the disabled entry from must-have list. Since we already updated it.
            delete $g_must_have_entires->{$key};
            $g_is_changed = 1;
            next;
        }

        # Remove the entry from must-have list. Since we already checked it.
        if (defined($g_must_have_entires->{$key}))
        {
            delete $g_must_have_entires->{$key};
            if ($is_marker_line)
            {
                # Record the setting lines added by Centrify.
                TRACE_OUT("Setting line added by Centrify is found: [$line]");
                push @g_wrapped_entries, $key;
            }
        }

        # Nothing to do if the required setting is correct.
        # TODO: Setting might have value list (a set of values), we need to 
        # make sure the required value is in the list.
        # Here, we will not handle the list value. Since we don't know it's
        # a single value or a value list.
        next if ($value eq $req_conf->{$key});

        # If we find a line containing setting with unexpected value,
        # just update the value inside the line.

        $line = $key;
        $line = "include" if ($is_include_schema_setting);

        # Fomat control:
        # key       value
        $line = GenerateSettingLine($line, $req_conf->{$key});
        $g_is_changed = 1;
    }
    continue
    {
        if ($line =~ /$SLAPD_CONF_START_PATTERN/)
        {
            TRACE_OUT("Matching Centrify start flag: [$line]");
            $is_marker_line = 1;
        }

        # For lines added by Centrify, we will not handle here.
        # Since we want the wrapped settings added by Centrify would be in
        # proper place of the conf file.
        # Will add them back in sub CheckSlapdConfiguration().
        if ($is_marker_line)
        {
            if ($line =~ /$SLAPD_CONF_END_PATTERN/)
            {
                TRACE_OUT("Matching Centrify end flag: [$line]");
                $is_marker_line = 0;
            }
        }
        else
        { 
            if (!$omitted_line)
            {
                push @new_lines, "$line";
                $line_num++;
            }
        }
    }

    return \@new_lines;
}

sub GenerateSettingLine($$)
{
    my $key = shift @_;
    my $value = shift @_;
    my $line = "$key\t";
    $line .= "\t" if (length($key) < 8);
    $line .= "$value";
    return $line;
}

#   $_[0]: $array - the array reference.
#   $_[1]: $index - the array index.
sub DeleteMemberFromArrayRef($$)
{
    my $array = shift;
    my $index = shift;
    splice(@$array, $index, 1);
}

#   $_[0]: $array - the array reference.
#   $_[1]: $index - the indexes array reference.
sub DeleteMembersFromArrayRef($$)
{
    my ($array, $index) = @_;
    # Make sure delete the last member firstly.
    foreach (sort {$b <=> $a} @$index)
    {
        DeleteMemberFromArrayRef($array, $_);
    }
}

#
# Check if CDC is in solely upgraded case.
# Note: old version slapd might not be compatible with new version CDC.
#
#   return: 0 - no, we can try to restart slapd server.
#           1 - yes, definitely not try to restart slapd server.
#
sub IsCDCSolelyUpgraded()
{
    # File SLAPD_VER will be installed while package CDC is upgrading.
    my $SLAPD_VERSION = "/var/centrifydc/upgrade/SLAPD_VER";
    my $cmd_check_cdcversion = ["/usr/bin/adinfo", '-v'];
    # We don't expect a symbol link.
    # If no file SLAPD_VER found, then we are not in the CDC upgrade case.
    # It's safe to restart slapd.
    return 0 if (! -f $SLAPD_VERSION);
    DEBUG_OUT("File $SLAPD_VERSION found. Centrify DirectControl might be in upgrade case.");

    # Get the CDC version
    my ($rc, $output) = RunCmd($cmd_check_cdcversion);
    # Don't restart slapd server If failed to check CDC version.
    return 1 if (!defined $rc or $rc ne '0');
    chomp $output;
    # adinfo -v: adinfo (CentrifyDC 5.2.3-379)
    my $VERSION_PATTERN = qr/CentrifyDC\s+(.*)-/;
    $g_cdcversion = ($output =~ /$VERSION_PATTERN/)[0];

    # Get the slapd version.
    $g_slapdversion = ReadFile("$SLAPD_VERSION");
    chomp $g_slapdversion;
    DEBUG_OUT("Centrify-ldapproxy version: [$g_slapdversion], Centrify DirectControl version: [$g_cdcversion].");

    # It is safe to restart slapd server ONLY when both slapd and CDC have the same
    # version.
    "$g_slapdversion" eq "$g_cdcversion" ? return 0 : return 1;
}

#
# Check and update slapd.conf.
# Load system settings from slapd.conf. Check if the required settings are
# correct. Fix the incorrect required settings and append the missing  required
# settings to the end of slapd.conf.
#
#   return: 0 - success
#           1 - failure
#
sub CheckSlapdConfiguration()
{
    InitMustHaveConfigEntries();

    my $flock = autoedit_file_lock::new("$g_cfg_name");
    if (!defined $flock)
    {
        DEBUG_OUT("Cannot create lock file for file $g_cfg_name");
        return 1;
    }

    my $conf_lines = LoadSystemConfiguration();
    return 1 if (!defined($conf_lines));
    foreach (keys %$g_must_have_entires)
    { 
        if (!$g_must_have_entires->{$_})
        {
            # Delete those settings if it's been explicitly disabled by cdc.conf
            # e.g. ldapproxy.require.authc: false
            TRACE_OUT("Slapd server setting [$_] is disabled by CentrifyDC configuration file.");
            delete $g_must_have_entires->{$_};
        }
    }

    # That's good if no setting is changed and missing.
    return 0 if (!scalar keys %$g_must_have_entires && !$g_is_changed);
    push @g_wrapped_entries, keys %$g_must_have_entires;
    my %tmp_entries = map {$_ => 1} @g_wrapped_entries;

    my @appending_lines;
    # Append the missing required database directives to conf file.
    push @appending_lines, "$SLAPD_CONF_DATABASE_START_MARKER";
    push @appending_lines, "$SLAPD_CONF_WARNING_MESSAGE";
 
    # If the "database" directive not found, we will append to the end.
    $g_database_index = scalar(@$conf_lines) if (!defined($g_database_index));
    my $is_database_directive_appended = 0;
    foreach my $key (SLAPD_DATABASE_SPECIFIC)
    {
        next if (!defined($tmp_entries{$key}));
        my $line = GenerateSettingLine($key, $g_req_conf->{$key});
        push @appending_lines, $line;
        $is_database_directive_appended = 1;
    }
    push @appending_lines, "\n$SLAPD_CONF_DATABASE_END_MARKER";

    # We don't want to insert an wrapped section without any valid setting
    # entry to the conf file.
    if ($is_database_directive_appended)
    {
        # Mark as changed. So we will restart the slapd server if it's running.
        $g_is_changed = 1;
        splice(@$conf_lines, $g_database_index + 1, 0, @appending_lines);
    }

    # Insert the missing required backend directives to conf file.
    @appending_lines = ();
    push @appending_lines, "$SLAPD_CONF_BACKEND_START_MARKER";
    push @appending_lines, "$SLAPD_CONF_WARNING_MESSAGE";
    my $is_backend_directive_changed = 0;
    foreach my $key (SLAPD_BACKEND_SPECIFIC)
    {
        next if (!defined($tmp_entries{$key}));
        my $line = GenerateSettingLine($key, $g_req_conf->{$key});
        push @appending_lines, $line;
        $is_backend_directive_changed= 1;
    }
    push @appending_lines, "\n$SLAPD_CONF_BACKEND_END_MARKER";
    if ($is_backend_directive_changed)
    {
        # Mark as changed. So we will restart the slapd server if it's running.
        $g_is_changed = 1;
        splice(@$conf_lines, $g_database_index, 0, @appending_lines);
    }

    # Insert the missing required global directives to conf file.
    @appending_lines = ();
    push @appending_lines, "$SLAPD_CONF_GLOBAL_START_MARKER";
    push @appending_lines, "$SLAPD_CONF_WARNING_MESSAGE";
    my $is_global_directive_changed = 0;
    # If any of the required schemas is missing, just rewrite the schemas lines.
    # So we can easily handle their orders.
    if (scalar(keys %g_global_index) < scalar(SLAPD_GLOBAL_SCHEMAS))
    {
        # Let's get the line number of the first schema in slapd.conf
        my @ascending_index = (sort {$a <=> $b} values %g_global_index);
        $g_global_index{'core'} = $ascending_index[0];
        # Delete all existing schemas. Then we can wrap them together.
        DeleteMembersFromArrayRef($conf_lines, \@ascending_index);
        # Insert all required schemas as a wrapped block to slapd.conf.
        # So we can easily handle the order.
        foreach my $key (SLAPD_GLOBAL_SCHEMAS)
        {
            my $line = GenerateSettingLine('include', $g_req_conf->{'include'}->{$key});
            push @appending_lines, $line;
        }
        $is_global_directive_changed= 1;
    }
    push @appending_lines, "\n$SLAPD_CONF_GLOBAL_END_MARKER";
    # If the "global" directive not found, we will insert to the beginning.
    $g_global_index{'core'} = 0 if (!defined($g_global_index{'core'}));

    if ($is_global_directive_changed)
    {
        # Mark as changed. So we will restart the slapd server if it's running.
        $g_is_changed = 1;
        # Insert the block at line "<full path>/core.schema". Note that this line
        # has been deleted. So we will not have duplicated lines of core.schema 
        # as we wrapped the core.schema in block.
        splice(@$conf_lines, $g_global_index{'core'}, 0, @appending_lines);
    }

    my $is_cdc_solely_upgraded = IsCDCSolelyUpgraded();
    # Don't try to restart slapd server. Since old version slapd might not be
    # compatible with new version CDC. Otherwise we might have slapd core dump.
    if ($is_cdc_solely_upgraded)
    {
        # We don't change the config file. So slapd server will not be restarted.
        my $msg = "Higher version Centrify DirectControl [$g_cdcversion] is detected. "
                . "Will restart Centrify-ldapproxy server [current version: $g_slapdversion] "
                . "till it has been upgraded to the same version.";
        INFO_OUT($msg);
        $g_is_changed = 0;
        return 0;
    }
    my $new_conf_content = join("\n", @$conf_lines) . "\n";
    if (!WriteFile($g_cfg_name, $new_conf_content))
    {
        ERROR_OUT("Cannot save changed settings to configuration file $g_cfg_name");
        return 1;
    }
    return 0;
}

#
# Function to restart the slapd server.
# Will restart the slapd server only when it is running.
#
#   return: 0 - success
#           1 - failure
#
sub RestartSlapdService()
{
    my $CENTRIFY_LDAPPROXY_SCRIPT = "$CIMS_INSTALL/bin/centrify-ldapproxy";
    my $cmd_check = ["$CENTRIFY_LDAPPROXY_SCRIPT", 'status'];
    my $cmd_restart = ["$CENTRIFY_LDAPPROXY_SCRIPT", 'restart'];

    # Ensure that the script used to restart slapd server exists.
    if (!-x $CENTRIFY_LDAPPROXY_SCRIPT)
    {
        ERROR_OUT("Failed to find Centrify ldapproxy script $CENTRIFY_LDAPPROXY_SCRIPT. Please manually restart Centrify ldapproxy server.");
        return 1;
    }
    my ($rc, $output) = RunCmd($cmd_check);
    if ($rc)
    {
        chomp $output;
        ERROR_OUT("Failed to get Centrify ldapproxy server status ($rc): [$output]. Please manually restart Centrify ldapproxy server.");
        return $rc;
    }

    $rc = 0;
    # Restart the slapd server only when it's running.
    if ($output !~ /is running/)
    {
        INFO_OUT("Centirfy ldapproxy server is not running. Will not restart Centrify ldapproxy server.");
        return $rc;
        
    }

    DEBUG_OUT("Restarting Centrify ldapproxy server.");
    ($rc, $output) = RunCmd($cmd_restart);
    ERROR_OUT("Fail to restart Centrify ldapproxy server ($rc): [$output]. Please manually restart Centrify ldapproxy server.") if ($rc);
    return $rc;
}

# >>> MAIN >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

GetOptions( "cfg_file=s" => \$g_cfg_name,
            ) or Usage;

# Try to create config file slapd.conf if not found.
if (!-f $g_cfg_name)
{
    # Assume file ldapproxy.slapd.conf does not exist if centrify-ldapproxy is
    # not installed.
    if (!-f LDAPPROXY_CFG_FILE)
    {
        # Nothing to do without centrify-ldapproxy installed.
        DEBUG_OUT("Centrify-ldapproxy is not installed. Ignore file $g_cfg_name and exit");
        exit 0;
    }
    my $flock = autoedit_file_lock::new(LDAPPROXY_CFG_FILE);
    if (!defined $flock)
    {
        DEBUG_OUT("Cannot create lock file for file LDAPPROXY_CFG_FILE");
        exit 1;
    }
    if (!CopyFile(LDAPPROXY_CFG_FILE, $g_cfg_name))
    {
        ERROR_OUT("Cannot create file $g_cfg_name");
        exit 1;
    }
}

my $real_name = readlink($g_cfg_name);
if (defined $real_name)
{
    my $real_name = TraverseSymLink($g_cfg_name);
    if (!defined $real_name)
    {
        ERROR_OUT("Failed to traverse symlink $g_cfg_name");
        exit 1;
    }
    $g_cfg_name = $real_name;
}

# Always back up before modifying.
my $backup = "$g_cfg_name.pre_cdc";
if (!CopyFile($g_cfg_name, $backup))
{
    ERROR_OUT("Cannot backup $g_cfg_name to $backup");
    exit 1;
}
my $rc = 0;
$rc = CheckSlapdConfiguration();
exit $rc if ($rc);

# Make sure slapd.conf file permission is 644
# chmod will return number of files successfully changed.
if (chmod(0644, $g_cfg_name) != 1)
{
    WARN_OUT("Cannot set file permission '0644' for $g_cfg_name: $rc");
    exit 1;
}

# Try to restart slapd server if configuration file is changed.
if ($g_is_changed)
{
    INFO_OUT("Configuration file $g_cfg_name has been changed successfully. Try to restart ldapproxy server");
    $rc = RestartSlapdService();
}

exit $rc;
